import 'dart:io';

import 'package:path/path.dart';
import 'package:protobuf3_parser/src/asset_manager/asset_id.dart';
import 'package:protobuf3_parser/src/asset_manager/asset_writer.dart';
import 'package:protobuf3_parser/src/extensions.dart';

class DirectoryAssetWriter extends AssetWriter {
  final Directory _dir;
  DirectoryAssetWriter(this._dir);

  File _getFile(AssetId id) {
    final file = join(_dir.path, id.path).file;
    if (!file.existsSync()) file.createSync(recursive: true);
    return file;
  }

  @override
  Future<void> writeBytes(AssetId id, List<int> bytes,
          {FileMode mode = FileMode.write}) =>
      _getFile(id).writeAsBytes(bytes, mode: mode);

  @override
  Future<void> writeStream(
    AssetId id,
    Stream<List<int>> stream, {
    FileMode mode = FileMode.write,
  }) async {
    final w = _getFile(id).openWrite(mode: mode);
    await w.addStream(stream);
    await w.flush();
    await w.close();
  }

  @override
  Future<void> writeString(AssetId id, String string,
          {FileMode mode = FileMode.write}) =>
      _getFile(id).writeAsString(string, mode: mode);

  @override
  Future<bool> exists(AssetId id) => _getFile(id).exists();
}
