import 'package:protobuf3_parser/protobuf3_parser.dart';

final minSrcGen = MiniSourceGenerator();

/// Minified source code generator.
// TODO: Generate comments.
class MiniSourceGenerator extends SourceGenerator {
  @override
  Future<String> forMessage(Message message) async {
    final x = StringBuffer();

    // message NAME {
    x.write("message ${message.name}{");

    // fields
    for (final field in message.fields) {
      x.write(await forMessageField(field));
    }

    // sub enums.
    for (final subEnum in message.subEnums) {
      x.write(await forEnum(subEnum));
    }

    // sub messages.
    for (final subMessage in message.subMessages) {
      x.write(await forMessage(subMessage));
    }

    // }
    x.write("}");

    return x.toString();
  }

  @override
  Future<String> forEnum(Enum enum_) async {
    final x = StringBuffer();

    // enum NAME {
    x.write("enum ${enum_.name}{");

    // fields
    for (final field in enum_.fields) {
      x.write(forEnumField(field));
    }

    x.write("}");

    return x.toString();
  }

  @override
  Future<String> forService(Service service) async {
    final x = StringBuffer();

    // service NAME {
    x.write("service ${service.name}{");

    // methods
    for (final method in service.methods) {
      x.write(await forServiceMethod(method));
    }

    // }
    x.write("}");
    return x.toString();
  }

  @override
  String forEnumField(EnumField field) => "${field.name}=${field.id};";

  @override
  Future<String> forMessageField(MessageField field) async =>
      "${(await field.type).name} ${field.name}=${field.id};";

  @override
  Future<String> forServiceMethod(ServiceMethod method) async {
    final x = StringBuffer();
    x.write("rpc ${method.name}(");
    if (method.isStreamRequest) {
      x.write("stream ");
    }
    x.write("${(await method.request).name})returns(");
    if (method.isStreamResponse) {
      x.write("stream ");
    }
    x.write("${(await method.response).name});");
    return x.toString();
  }
}
