import 'dart:async';

import 'package:protobuf3_parser/protobuf3_parser.dart';

abstract class SourceGenerator {
  Future<String> forElement(Element x) async {
    if (x is Message) {
      return await forMessage(x);
    } else if (x is MessageField) {
      return await forMessageField(x);
    } else if (x is Enum) {
      return await forEnum(x);
    } else if (x is EnumField) {
      return await forEnumField(x);
    } else if (x is Service) {
      return await forService(x);
    } else if (x is ServiceMethod) {
      return await forServiceMethod(x);
    }

    throw Exception(
        "Couldn't generate source for element type ${x.runtimeType}!");
  }

  /// Generate source for [Message].
  FutureOr<String> forMessage(Message message);

  /// Generate source for [MessageField].
  FutureOr<String> forMessageField(MessageField field);

  /// Generate source for [Enum].
  FutureOr<String> forEnum(Enum enum_);

  /// Generate source for [EnumField].
  FutureOr<String> forEnumField(EnumField field);

  /// Generate source for [Service].
  FutureOr<String> forService(Service service);

  /// Generate source for [ServiceMethod].
  FutureOr<String> forServiceMethod(ServiceMethod method);
}
