import 'package:protobuf3_parser/__generated__/antlr/Protobuf3Parser.dart';
import 'package:protobuf3_parser/protobuf3_parser.dart';

typedef Enums = List<Enum>;
typedef Messages = List<Message>;
typedef MessageFields = List<MessageField>;

class Message extends Element<MessageDefContext, Message> {
  Message(MessageDefContext ctx, Parser parser) : super(ctx, parser);

  @override
  String get name => ctx.messageName()!.text;

  @override
  Message? get parentElement {
    final parent = ctx.parent?.parent?.parent;

    if (parent is MessageDefContext) {
      return Message(parent, parser);
    } else {
      return null;
    }
  }

  Messages get subMessages {
    final l = <Message>[];

    final elements = ctx.messageBody()?.messageElements();
    if (elements != null) {
      for (final el in elements) {
        final messageDef = el.messageDef();
        if (messageDef != null) {
          l.add(Message(messageDef, parser));
        }
      }
    }

    return l;
  }

  Enums get subEnums {
    final l = <Enum>[];

    final elements = ctx.messageBody()?.messageElements();
    if (elements != null) {
      for (final el in elements) {
        final enumDef = el.enumDef();
        if (enumDef != null) {
          l.add(Enum(enumDef, parser));
        }
      }
    }

    return l;
  }

  MessageFields get fields {
    final l = <MessageField>[];

    final elements = ctx.messageBody()?.messageElements();
    if (elements != null) {
      for (final el in elements) {
        final field = el.field();
        if (field != null) {
          l.add(MessageField(field, parser));
        }
      }
    }

    return l;
  }

  @override
  Comments get comments => parser.convertToComments(ctx.comments());
}

class CoreTypeMessage extends Message {
  @override
  String name;

  CoreTypeMessage(MessageDefContext ctx, this.name, Parser parser)
      : super(ctx, parser);
}
