import 'package:antlr4/antlr4.dart' as antlr;
import 'package:protobuf3_parser/__generated__/antlr/Protobuf3Parser.dart';
import 'package:protobuf3_parser/protobuf3_parser.dart';
import 'package:protobuf3_parser/src/parser/core_types.g.dart';

class MessageField extends Element<FieldContext, Message> {
  MessageField(FieldContext ctx, Parser parser) : super(ctx, parser);

  @override
  String get name => ctx.fieldName()!.text;

  @override
  Message? get parentElement {
    final parent = ctx.parent?.parent?.parent;

    if (parent is MessageDefContext) {
      return Message(parent, parser);
    } else {
      return null;
    }
  }

  int get id => int.parse(ctx.fieldNumber()!.text);

  bool get isRepeated => ctx.REPEATED() != null;

  Future<Element<antlr.ParserRuleContext, dynamic>> get type async {
    final typeStr = ctx.type_()!.text;

    // search in the parent message
    final parent = parentElement;
    if (parent != null) {
      if (parent.name == typeStr) {
        return parent;
      }

      for (final message in parent.subMessages) {
        if (message.name == typeStr) {
          return message;
        }
      }
      for (final enum_ in parent.subEnums) {
        if (enum_.name == typeStr) {
          return enum_;
        }
      }
    }

    // search for messages and enums in other toplevel files
    await for (final file in parser.files) {
      await for (final message in file.messages) {
        if (message.name == typeStr) {
          return message;
        }
      }
    }
    await for (final file in parser.files) {
      await for (final enum_ in file.enums) {
        if (enum_.name == typeStr) {
          return enum_;
        }
      }
    }

    // check if it could be a core type
    if (CoreTypes.values.contains(typeStr)) {
      //! TODO
      return CoreTypeMessage(parent!.ctx, typeStr, parser);
    }

    // return this;
    throw "Couldn't find definition for the type $typeStr!";
  }

  @override
  Comments get comments => parser.convertToComments(ctx.comments());
}
