from os.path import abspath, dirname, join, exists
from os import makedirs, system, walk
from shutil import rmtree, which
from argparse import ArgumentParser


ANTLR_SEARCH_PATHS  = [
    "/usr/local/lib/"
]

JAVA            = which("java")

THIS_DIR        = dirname(abspath(__file__))
ROOT_DIR        = dirname(THIS_DIR)

GRAMMAR_FILE    = join(ROOT_DIR, "grammar/Protobuf3.g4")
GEN_DIR         = join(ROOT_DIR, "lib/__generated__/antlr")

if exists(GEN_DIR): rmtree(GEN_DIR)
makedirs(GEN_DIR)


def search_for_antlr(path: str) -> str | None:
    for root, _, files in walk(path):
        for file_name in files:
            if file_name.startswith("antlr") and file_name.endswith(".jar"):
                return join(root, file_name)
    return None



parser = ArgumentParser()
parser.add_argument("--antlr")
args = parser.parse_args()

antlr_path = args.antlr

if not antlr_path:
    for path in ANTLR_SEARCH_PATHS:
        x = search_for_antlr(path)
        if x:
            antlr_path = x
            break

if not antlr_path:
    print("Couldn't find antlr in:")
    for path in ANTLR_SEARCH_PATHS:
        print("-", path)
    exit(1)


ANTLR           = f"java -Xmx500M -cp '{antlr_path}:$CLASSPATH' org.antlr.v4.Tool"

system(f"{ANTLR} -Dlanguage=Dart -o {GEN_DIR} {GRAMMAR_FILE}")